#!/bin/bash -i

###Converted from AttractEasyModel by easy2model 1.3
### Generated by ATTRACT shell script generator version 0.8

set -u -e
if [ ! -f $ATTRACTDIR/attract ]; then
  echo 'Cannot find the ATTRACT binary in $ATTRACTDIR='\"$ATTRACTDIR\"
  echo 'Please install ATTRACT following the installation instructions'
  echo 'ATTRACT is available at: http://www.attract.ph.tum.de/services/ATTRACT/attract.tgz'
  exit 1
fi
if [ ! -f $ATTRACTDIR/../version ] ||  [ `awk '{print ($1 < 0.4)}' $ATTRACTDIR/../version` -eq 1 ]; then
  echo 'Your ATTRACT version is too old to run this protocol. Please download and install the latest version of ATTRACT'
  echo 'ATTRACT is available at: http://www.attract.ph.tum.de/services/ATTRACT/attract.tgz'
  exit 1
fi
trap "kill -- -$BASHPID; attractclean" ERR EXIT
$ATTRACTDIR/shm-clean

rm -rf result.dat result.pdb result.lrmsd result.irmsd result.fnat result.interface >& /dev/null
function attractclean {
  $ATTRACTDIR/shm-clean
}

#name of the run
name=protein_3_ens_rna_rigid

#docking parameters
params="$ATTRACTDIR/../attract.par partner1-ensemble/model-1r.pdb ligandr.pdb --fix-receptor --ens 1 partner1-ensemble.list --gravity 2"
scoreparams="$ATTRACTDIR/../attract.par partner1-ensemble/model-1r.pdb ligandr.pdb --score --fix-receptor --ens 1 partner1-ensemble.list"

#grid parameters
gridparams=" --grid 1 receptorgrid.gridheader"

#parallelization parameters
parals="--np 4 --chunks 4"

#see if pypy is installed
PYPY=python2
command -v pypy >/dev/null 2>&1 && PYPY=pypy

if [ 1 -eq 1 ]; then ### move and change to disable parts of the protocol

echo '**************************************************************'
echo 'Reduce partner PDBs...'
echo '**************************************************************'
rm -rf partner1-ensemble
mkdir partner1-ensemble
$ATTRACTTOOLS/splitmodel receptor.pdb partner1-ensemble/model 3 > /dev/null
cat /dev/null > partner1-ensemble.list
cat /dev/null > partner1-ensemble-aa.list
cat /dev/null > partner1-ensemble-aa-rmsd.list
python2 $ATTRACTDIR/../allatom/aareduce.py partner1-ensemble/model-1.pdb partner1-ensemble/model-1-aa.pdb --chain A --dumppatch --pdb2pqr > receptor.mapping
python2 $ATTRACTDIR/../allatom/aareduce.py partner1-ensemble/model-1-aa.pdb partner1-ensemble/model-1-heavy.pdb --heavy  --readpatch  > /dev/null
python2 $ATTRACTTOOLS/reduce.py partner1-ensemble/model-1-aa.pdb partner1-ensemble/model-1r.pdb --chain A > /dev/null
echo partner1-ensemble/model-1r.pdb >> partner1-ensemble.list
echo partner1-ensemble/model-1-aa.pdb >> partner1-ensemble-aa.list
echo partner1-ensemble/model-1-heavy.pdb >> partner1-ensemble-aa-rmsd.list
python2 $ATTRACTDIR/../allatom/aareduce.py partner1-ensemble/model-2.pdb partner1-ensemble/model-2-aa.pdb --chain A --dumppatch --reference partner1-ensemble/model-1-aa.pdb --pdb2pqr > /dev/null
python2 $ATTRACTDIR/../allatom/aareduce.py partner1-ensemble/model-2-aa.pdb partner1-ensemble/model-2-heavy.pdb --heavy  --readpatch  > /dev/null
python2 $ATTRACTTOOLS/reduce.py partner1-ensemble/model-2-aa.pdb partner1-ensemble/model-2r.pdb --chain A > /dev/null
echo partner1-ensemble/model-2r.pdb >> partner1-ensemble.list
echo partner1-ensemble/model-2-aa.pdb >> partner1-ensemble-aa.list
echo partner1-ensemble/model-2-heavy.pdb >> partner1-ensemble-aa-rmsd.list
python2 $ATTRACTDIR/../allatom/aareduce.py partner1-ensemble/model-3.pdb partner1-ensemble/model-3-aa.pdb --chain A --dumppatch --reference partner1-ensemble/model-1-aa.pdb --pdb2pqr > /dev/null
python2 $ATTRACTDIR/../allatom/aareduce.py partner1-ensemble/model-3-aa.pdb partner1-ensemble/model-3-heavy.pdb --heavy  --readpatch  > /dev/null
python2 $ATTRACTTOOLS/reduce.py partner1-ensemble/model-3-aa.pdb partner1-ensemble/model-3r.pdb --chain A > /dev/null
echo partner1-ensemble/model-3r.pdb >> partner1-ensemble.list
echo partner1-ensemble/model-3-aa.pdb >> partner1-ensemble-aa.list
echo partner1-ensemble/model-3-heavy.pdb >> partner1-ensemble-aa-rmsd.list
python2 $ATTRACTDIR/../allatom/aareduce.py ligand.pdb ligand-aa.pdb --chain B --rna --dumppatch --nalib > ligand.mapping
python2 $ATTRACTDIR/../allatom/aareduce.py ligand-aa.pdb ligand-heavy.pdb --heavy --chain B --rna --readpatch  > /dev/null
python2 $ATTRACTTOOLS/reduce.py ligand-aa.pdb ligandr.pdb --chain B --rna > /dev/null

echo '**************************************************************'
echo 'Reduce reference PDBs...'
echo '**************************************************************'
python2 $ATTRACTDIR/../allatom/aareduce.py receptor-rmsd.pdb refe-rmsd-1.pdb --heavy --pdb2pqr > /dev/null
python2 $ATTRACTDIR/../allatom/aareduce.py ligand-rmsd.pdb refe-rmsd-2.pdb --heavy --nalib  --rna > /dev/null

echo '**************************************************************'
echo 'Generate starting structures...'
echo '**************************************************************'
cat $ATTRACTDIR/../rotation.dat > rotation.dat
$ATTRACTDIR/translate partner1-ensemble/model-1r.pdb ligandr.pdb > translate.dat
$ATTRACTDIR/systsearch > systsearch.dat
start=systsearch.dat

echo '**************************************************************'
echo 'ensemble search:'
echo ' add ensemble conformations to the starting structures'
echo '**************************************************************'

echo '**************************************************************'
echo 'random ensemble conformation in ligand 1 for each starting structure'
echo '**************************************************************'
$PYPY $ATTRACTTOOLS/ensemblize.py systsearch.dat 3 1 random  > systsearch-ens1.dat

echo '**************************************************************'
echo 'calculate receptorgrid grid'
echo '**************************************************************'
awk '{print substr($0,58,2)}' ligandr.pdb | sort -nu > receptorgrid.alphabet
$ATTRACTDIR/make-grid-omp partner1-ensemble/model-1r.pdb $ATTRACTDIR/../attract.par 10.0 12.0 receptorgrid.gridheader  --shm --alphabet receptorgrid.alphabet

echo '**************************************************************'
echo 'Docking'
echo '**************************************************************'

echo '**************************************************************'
echo '1st minimization'
echo '**************************************************************'
python2 $ATTRACTDIR/../protocols/attract.py systsearch-ens1.dat $params $gridparams --vmax 1000 $parals --output out_$name.dat

echo '**************************************************************'
echo 'Final rescoring'
echo '**************************************************************'
python2 $ATTRACTDIR/../protocols/attract.py out_$name.dat $scoreparams --rcut 50.0 $parals --output out_$name.score

echo '**************************************************************'
echo 'Merge the scores with the structures'
echo '**************************************************************'
$PYPY $ATTRACTTOOLS/fill-energies.py out_$name.dat out_$name.score > out_$name-scored.dat

echo '**************************************************************'
echo 'Sort structures'
echo '**************************************************************'
$PYPY $ATTRACTTOOLS/sort.py out_$name-scored.dat > out_$name-sorted.dat

$ATTRACTTOOLS/top out_$name-sorted.dat 100 > out_$name-top.dat

echo '**************************************************************'
echo 'Remove redundant structures'
echo '**************************************************************'
$ATTRACTDIR/deredundant out_$name-top.dat 2 --ens 3 0 | $PYPY $ATTRACTTOOLS/fill-deredundant.py /dev/stdin out_$name-top.dat > out_$name-top-dr.dat

echo '**************************************************************'
echo 'Soft-link the final results'
echo '**************************************************************'
ln -s out_$name-top-dr.dat result.dat

echo '**************************************************************'
echo 'collect top 10 structures:'
echo '**************************************************************'
$ATTRACTTOOLS/top out_$name-top-dr.dat 10 > out_$name-top10.dat
$ATTRACTDIR/collect out_$name-top10.dat partner1-ensemble/model-1-aa.pdb ligand-aa.pdb --ens 1 partner1-ensemble-aa.list > out_$name-top10.pdb
ln -s out_$name-top10.pdb result.pdb

echo '**************************************************************'
echo 'calculate all-atom interface RMSD'
echo '**************************************************************'
python2 $ATTRACTDIR/irmsd.py out_$name-top-dr.dat partner1-ensemble/model-1-heavy.pdb refe-rmsd-1.pdb ligand-heavy.pdb refe-rmsd-2.pdb --ens 1 partner1-ensemble-aa-rmsd.list --allatoms > out_$name-top-dr.irmsd
ln -s out_$name-top-dr.irmsd result.irmsd

attractclean

fi ### move to disable parts of the protocol
